import { ElementRef, Injectable, SecurityContext } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import Prism from 'prismjs';
import * as _ from 'lodash-es';
export class PrismService {
    constructor(sanitizer) {
        this.sanitizer = sanitizer;
        this.prism = Prism;
        this.templateOptions = { interpolate: /{{([\s\S]+?)}}/g };
    }
    highlight(codeElementRef, options) {
        if (codeElementRef instanceof ElementRef) {
            if (options.code !== undefined) {
                codeElementRef.nativeElement.innerHTML = this.sanitizer.sanitize(SecurityContext.HTML, this.escapeHtml(options.code));
            }
            if (options.interpolation) {
                this.interpolate(codeElementRef, options.interpolation);
            }
            this.prism.highlightElement(codeElementRef.nativeElement, options.async, options.callback);
        }
    }
    hooks() {
        return this.prism.hooks;
    }
    escapeHtml(unsafe) {
        return unsafe
            .replace(/&/g, '&amp;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#039;');
    }
    interpolate(elementRef, interpolation) {
        if (interpolation && typeof interpolation === 'object') {
            return elementRef.nativeElement.innerHTML = _.template(elementRef.nativeElement.innerHTML, this.templateOptions)(interpolation);
        }
        return elementRef.nativeElement.innerHTML;
    }
}
PrismService.decorators = [
    { type: Injectable },
];
PrismService.ctorParameters = () => [
    { type: DomSanitizer, },
];
//# sourceMappingURL=prism.service.js.map